@@ -64,6 +64,8 @@ release_op_data (void *hook)
 
       if (sig->fpr)
 	free (sig->fpr);
+      if (sig->pka_address)
+	free (sig->pka_address);
       free (sig);
       sig = next;
     }
@@ -434,7 +436,7 @@ parse_notation (gpgme_signature_t sig, gpgme_status_code_t code, char *args)
 
       if (code == GPGME_STATUS_NOTATION_NAME)
 	{
-	  err = _gpgme_decode_percent_string (args, &notation->name, 0);
+	  err = _gpgme_decode_percent_string (args, &notation->name, 0, 0);
 	  if (err)
 	    {
 	      _gpgme_sig_notation_free (notation);
@@ -453,7 +455,7 @@ parse_notation (gpgme_signature_t sig, gpgme_status_code_t code, char *args)
 	{
 	  /* This is a policy URL.  */
 
-	  err = _gpgme_decode_percent_string (args, &notation->value, 0);
+	  err = _gpgme_decode_percent_string (args, &notation->value, 0, 0);
 	  if (err)
 	    {
 	      _gpgme_sig_notation_free (notation);
@@ -497,7 +499,7 @@ parse_notation (gpgme_signature_t sig, gpgme_status_code_t code, char *args)
 	  dest += cur_len;
 	}
       
-      err = _gpgme_decode_percent_string (args, &dest, len);
+      err = _gpgme_decode_percent_string (args, &dest, len, 0);
       if (err)
 	return err;
 
@@ -588,6 +590,7 @@ _gpgme_verify_status_handler (void *priv, gpgme_status_code_t code, char *args)
   void *hook;
   op_data_t opd;
   gpgme_signature_t sig;
+  char *end;
 
   err = _gpgme_op_data_lookup (ctx, OPDATA_VERIFY, &hook, -1, NULL);
   opd = hook;
@@ -651,6 +654,20 @@ _gpgme_verify_status_handler (void *priv, gpgme_status_code_t code, char *args)
       return sig ? parse_trust (sig, code, args)
 	: gpg_error (GPG_ERR_INV_ENGINE);
 
+    case GPGME_STATUS_PKA_TRUST_BAD:
+    case GPGME_STATUS_PKA_TRUST_GOOD:
+      opd->only_newsig_seen = 0;
+      /* Check that we only get one of these status codes per
+         signature; if not the crypto backend misbehaves.  */
+      if (!sig || sig->pka_trust || sig->pka_address)
+        return gpg_error (GPG_ERR_INV_ENGINE);
+      sig->pka_trust = code == GPGME_STATUS_PKA_TRUST_GOOD? 2 : 1;
+      end = strchr (args, ' ');
+      if (end)
+        *end = 0;
+      sig->pka_address = strdup (args);
+      break;
+
     case GPGME_STATUS_ERROR:
       opd->only_newsig_seen = 0;
       /* The error status is informational, so we don't return an
